'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.toPublishOptions = toPublishOptions;
exports.publish = publish;

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _options = require('../utils/options');

var options = _interopRequireWildcard(_options);

var _errors = require('../utils/errors');

var _logger = require('../utils/logger');

var logger = _interopRequireWildcard(_logger);

var _messages = require('../utils/messages');

var messages = _interopRequireWildcard(_messages);

var _locks = require('../utils/locks');

var locks = _interopRequireWildcard(_locks);

var _npm = require('../utils/npm');

var npm = _interopRequireWildcard(_npm);

var _Project = require('../Project');

var _Project2 = _interopRequireDefault(_Project);

var _Package = require('../Package');

var _Package2 = _interopRequireDefault(_Package);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toPublishOptions(args, flags) {
  return {
    cwd: options.string(flags.cwd, 'cwd'),
    access: options.string(flags.access, 'access')
  };
}


async function getUnpublishedPackages(packages) {
  let semverGtCheckFailWithWarning = (pkgLocalVersion, pkgPublishedVersion, pkgName) => {
    let shouldPublish = _semver2.default.gt(pkgLocalVersion, pkgPublishedVersion);

    // show a warning message if package is not published since pkgPublishedVersion > pkgLocalVersion
    if (!shouldPublish) {
      logger.warn(messages.notPublishingPackage(pkgLocalVersion, pkgPublishedVersion, pkgName));
    }

    return shouldPublish;
  };

  let results = await _promise2.default.all(packages.map(async pkg => {
    let config = pkg.config;
    let response = await npm.infoAllow404(config.getName());

    return {
      name: config.getName(),
      localVersion: config.getVersion(),
      isPublished: response.published,
      newVersion: response.pkgInfo.version || ''
    };
  }));

  return results.filter(result => {
    // only publish if our version is higher than the one on npm
    return !result.isPublished || semverGtCheckFailWithWarning(result.localVersion, result.newVersion, result.name);
  });
}

async function publish(opts) {
  let cwd = opts.cwd || process.cwd();
  let project = await _Project2.default.init(cwd);
  let packages = await project.getPackages();
  let publicPackages = packages.filter(pkg => !pkg.config.getPrivate());
  let publishedPackages = [];

  try {
    // TODO: Re-enable once locking issues are sorted out
    // await locks.lock(packages);
    let unpublishedPackagesInfo = await getUnpublishedPackages(packages);
    let unpublishedPackages = packages.filter(pkg => {
      return unpublishedPackagesInfo.some(p => pkg.getName() === p.name);
    });

    if (unpublishedPackagesInfo.length === 0) {
      logger.warn(messages.noUnpublishedPackagesToPublish());
    }

    await project.runPackageTasks(unpublishedPackages, async pkg => {
      let name = pkg.config.getName();
      let version = pkg.config.getVersion();
      logger.info(messages.publishingPackage(name, version));

      let publishConfirmation = await npm.publish(name, {
        cwd: pkg.dir,
        access: opts.access
      });

      publishedPackages.push({
        name,
        newVersion: version,
        published: publishConfirmation && publishConfirmation.published
      });
    });

    return publishedPackages;

    // TODO: Re-enable once locking issues are sorted out
    // await locks.unlock(packages);
  } catch (err) {
    logger.error(err.message);
    throw new _errors.BoltError('Failed to publish');
  }
}